using System;
using System.Collections.Generic;
using System.Text;

namespace FmodEx {
	class Fmod : IDisposable {

		private readonly FMOD.System system;

		public Fmod() {
			ThrowExceptionForFmodResult(FMOD.Factory.System_Create(ref this.system));
			ThrowExceptionForFmodResult(this.system.init(100, FMOD.INITFLAG.NORMAL, IntPtr.Zero));
		}

		public void Update() {
			ThrowExceptionForFmodResult(this.system.update());
		}

		private static void ThrowExceptionForFmodResult(FMOD.RESULT r) {
			if (r != FMOD.RESULT.OK) throw new ApplicationException(FMOD.Error.String(r));
		}

		public void Dispose() {
			if (system != null) ThrowExceptionForFmodResult(system.release());
		}

		/// <summary>
		/// A sound file.
		/// </summary>
		public class Sound {

			private readonly Fmod system;
			private readonly FMOD.Sound sound;
			private FMOD.Channel channel;

			public Sound(Fmod system, string filename, bool stream) {
				this.system = system;
				Fmod.ThrowExceptionForFmodResult(
					stream ? system.system.createStream(filename, FMOD.MODE.DEFAULT, ref this.sound)
						   : system.system.createSound(filename, FMOD.MODE.DEFAULT, ref this.sound));
			}

			/// <summary>
			/// Play the sound.
			/// </summary>
			/// <param name="paused">Set true to set the keep ready to play, but paused.</param>
			public void Play(bool paused) {
				this.system.system.playSound(FMOD.CHANNELINDEX.FREE, this.sound, paused, ref this.channel);
			}

			/// <summary>
			/// Play the sound immediately.
			/// </summary>
			public void Play() {
				this.Play(false);
			}

			/// <summary>
			/// Gets or sets whether the song is paused.
			/// </summary>
			/// <remarks>You must have called Play prior to changing this property.</remarks>
			public bool Paused {
				get {
					bool b = default(bool);
					Fmod.ThrowExceptionForFmodResult(this.channel.isPlaying(ref b));
					return !b;
				}
				set {
					Fmod.ThrowExceptionForFmodResult(this.channel.setPaused(value));
				}
			}

			public TimeSpan Position {
				get {
					uint Time = default(uint);
					Fmod.ThrowExceptionForFmodResult(this.channel.getPosition(ref Time, FMOD.TIMEUNIT.MS));
					return TimeSpan.FromMilliseconds(Time);
				}
			}

			/// <summary>
			/// Mute or unmute the sound.
			/// </summary>
			/// <param name="muted">True to mute the sound; false to unmute it.</param>
			public void Mute(bool muted) {
				Fmod.ThrowExceptionForFmodResult(this.channel.setVolume(muted ? 0f : 1f));
			}

			public bool IsMuted {
				get {
					float f = default(float);
					Fmod.ThrowExceptionForFmodResult(this.channel.getVolume(ref f));
					return f < 0.5f;
				}
			}
		}

	}
}
