using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace Frets.Game {
	partial class SoundEffects {
		public class SoundEffect {

			/// <summary>
			/// The various different sounds in the effect.
			/// </summary>
			private readonly FmodEx.Fmod.Sound[] AllSounds;

			/// <summary>
			/// The FMOD sound system we're playing on.
			/// </summary>
			private readonly FmodEx.Fmod SoundSystem;

			/// <summary>
			/// Index of the last played sound.
			/// </summary>
			private int LastPlayedSound = 0;

			/// <summary>
			/// Create an instance of the sound effect class.
			/// </summary>
			/// <param name="soundSystem">The sound system to play the sound on.</param>
			/// <param name="baseDirectory">The base directory to load from.</param>
			/// <param name="effectName">The base name of the effect.</param>
			/// <remarks>For example using effect name "crunch" will load sounds crunch1.ogg, crunch2.ogg, ... crunchN.ogg from the base directory.</remarks>
			public SoundEffect(FmodEx.Fmod soundSystem, string baseDirectory, string effectName) {
				this.SoundSystem = soundSystem;

				// Search for matching effect names and load them.
				List<FmodEx.Fmod.Sound> LoadingSounds = new List<FmodEx.Fmod.Sound>();
				for (int i = 0; i < int.MaxValue; ++i) {
					string PotentialName = Path.Combine(baseDirectory, string.Format("{0}{1}.ogg", effectName, i == 0 ? "" : i.ToString()));
					if (File.Exists(PotentialName)) {
						LoadingSounds.Add(new FmodEx.Fmod.Sound(this.SoundSystem, PotentialName, true));
					} else if (i != 0) {
						break;
					}
				}


				this.AllSounds = LoadingSounds.ToArray();

				// Validate:
				if (AllSounds.Length == 0) throw new ArgumentException("Could not find any effects named " + effectName + ".");

			}

			/// <summary>
			/// Play the sound effect.
			/// </summary>
			public void Play() {
				AllSounds[LastPlayedSound++].Play();
				LastPlayedSound %= AllSounds.Length;
			}


		}
	}
}
